/*
 * Copyright (c) 2006-2009 OrangeSignal.com All rights reserved.
 */

package jp.sourceforge.orangesignal.trading.strategy.stop;

import java.util.LinkedList;

import jp.sourceforge.orangesignal.trading.Position;
import jp.sourceforge.orangesignal.trading.strategy.StrategyStatus;
import jp.sourceforge.orangesignal.trading.strategy.StrategySupport;

/**
 * ポジションの保有期間が、指定された期間を経過した場合に、
 * 成行注文で決済するロスカットルールのストラテジーを提供します。
 * 
 * @author 杉澤 浩二
 */
public class InactivityStop extends StrategySupport {

	/**
	 * 期間を保持します。
	 */
	private int period = 5;

	/**
	 * 期間を返します。
	 * 
	 * @return 期間
	 */
	public int getPeriod() { return period; }

	/**
	 * 期間を設定します。
	 * 
	 * @param period 期間
	 */
	public void setPeriod(final int period) { this.period = period; }

	@Override
	public StrategyStatus process() {
		// ポジション毎に経過期間を検証します。
		final LinkedList<Position> positions = getCurrentPositions();
		for (final Position p : positions) {
			final int hold = getCurrentDataIndex() - getDataset().indexOf(p.getEntryDate());
			if (hold >= period) {
				switch (p.getType()) {
					case LONG:
						sell(getStrategyName());
						break;
					case SHORT:
						buyToCover(getStrategyName());
						break;
				}
			}
		}
		return StrategyStatus.NEXT;
	}

}
