/*
 * Copyright (c) 2006-2009 OrangeSignal.com All rights reserved.
 * 
 * これは Apache ライセンス Version 2.0 (以下、このライセンスと記述) に
 * 従っています。このライセンスに準拠する場合以外、このファイルを使用
 * してはなりません。このライセンスのコピーは以下から入手できます。
 * 
 * http://www.apache.org/licenses/LICENSE-2.0.txt
 * 
 * 適用可能な法律がある、あるいは文書によって明記されている場合を除き、
 * このライセンスの下で配布されているソフトウェアは、明示的であるか暗黙の
 * うちであるかを問わず、「保証やあらゆる種類の条件を含んでおらず」、
 * 「あるがまま」の状態で提供されるものとします。
 * このライセンスが適用される特定の許諾と制限については、このライセンス
 * を参照してください。
 */

package jp.sf.orangesignal.chart.ui.screen;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import jp.sf.orangesignal.chart.ChartSettings;
import jp.sf.orangesignal.chart.ChartSettings.ShinneSettings;
import jp.sf.orangesignal.chart.event.ChartEvent;
import jp.sf.orangesignal.chart.event.ChartScreenEvent;
import jp.sf.orangesignal.chart.ui.Icons;
import jp.sf.orangesignal.chart.ui.canvas.IndexCanvas;
import jp.sf.orangesignal.chart.util.StringManager;

/**
 * 新値足サイド画面を提供します。
 * 
 * @author 杉澤 浩二
 */
public class ShinneSideScreen extends SideScreen {

	private static final long serialVersionUID = -3480283882947351261L;

	/**
	 * 指標キャンバスを保持します。
	 */
	private final IndexCanvas canvas;

	/**
	 * [範囲に0を含める]チェックボックスを保持します。
	 */
	private final JCheckBox fixed = new JCheckBox(StringManager.getString("fixed"));

	private JSpinner reversal = new JSpinner(new SpinnerNumberModel(1, 1, 30, 1));

	/**
	 * コンストラクタです。
	 * 
	 * @param icons アイコン情報
	 */
	public ShinneSideScreen(final Icons icons) {
		// 各コントロールを初期化します。
		this.canvas = new IndexCanvas(icons);

		// レイアウトを処理します。
		final GridBagLayout layout = new GridBagLayout();
		final GridBagConstraints c = new GridBagConstraints();
		setLayout(layout);
		int y = 0;

		addCanvas(layout, c, y++, this.canvas);
		addCheckBox(layout, c, y++, this.fixed);
		addSeparator(layout, c, y++);
		addPanel(layout, c, y, createParamPanel());
	}

	private JPanel createParamPanel() {
		final JPanel panel = new JPanel();
		final GridBagLayout layout = new GridBagLayout();
		panel.setLayout(layout);

		addSubSpinner(panel, layout, new GridBagConstraints(), 0, new JLabel(StringManager.getString("reversal")), this.reversal);

		return panel;
	}

	@Override
	public void load(final ChartSettings settings) {
		setVisibleAdvancedOptions(settings.advancedOptions);
		final ShinneSettings s = settings.shinne;
		this.fixed.setSelected(s.fixed);
		this.reversal.setValue(s.reversal);
	}

	@Override
	public void save(final ChartSettings settings) {
		final ShinneSettings s = settings.shinne;
		s.fixed = this.fixed.isSelected();
		s.reversal = (Integer) this.reversal.getValue();
	}

	public boolean isIgnoreStart(final ChartSettings settings) {
		final ShinneSettings s = settings.shinne;
		return s.fixed != this.fixed.isSelected();
	}

	@Override
	public void datasetSwitch(final ChartEvent e) {
		super.notify = false;
		final boolean enabled = e.getDataset() != null;

		this.fixed.setEnabled(enabled);
		this.reversal.setEnabled(enabled);

		if (enabled)
			load(e.getSettings());

		this.canvas.switchDataset(e);
		super.notify = true;
	}

	@Override
	public void periodSwitch(final ChartEvent e) {
		this.canvas.switchDataset(e);
	}

	@Override
	public void settingChanged(final ChartEvent e) {
		this.canvas.switchDataset(e);
	}

	@Override
	public void positionChanged(final ChartScreenEvent e) {
		this.canvas.setup(e);
	}

	@Override
	public void setVisibleAdvancedOptions(final boolean visible) {
		this.fixed.setVisible(visible);
	}

}
