/*
 * Copyright (c) 2006-2009 OrangeSignal.com All rights reserved.
 * 
 * これは Apache ライセンス Version 2.0 (以下、このライセンスと記述) に
 * 従っています。このライセンスに準拠する場合以外、このファイルを使用
 * してはなりません。このライセンスのコピーは以下から入手できます。
 * 
 * http://www.apache.org/licenses/LICENSE-2.0.txt
 * 
 * 適用可能な法律がある、あるいは文書によって明記されている場合を除き、
 * このライセンスの下で配布されているソフトウェアは、明示的であるか暗黙の
 * うちであるかを問わず、「保証やあらゆる種類の条件を含んでおらず」、
 * 「あるがまま」の状態で提供されるものとします。
 * このライセンスが適用される特定の許諾と制限については、このライセンス
 * を参照してください。
 */

package jp.sf.orangesignal.trading.strategy;

import java.util.Date;

import jp.sf.orangesignal.trading.Trader;
import jp.sf.orangesignal.trading.data.Dataset;
import jp.sf.orangesignal.trading.order.OrderTiming;

/**
 * トレーディングストラテジーのインターフェースを提供します。
 * 
 * @author 杉澤 浩二
 */
public interface TradingStrategy {

//	/**
//	 * 売買管理オブジェクトを返します。
//	 * 
//	 * @return 売買管理オブジェクト
//	 */
//	public Trader getTrader();

	/**
	 * 売買管理オブジェクトを設定します。
	 * 
	 * @param trader 売買管理オブジェクト
	 */
	public void setTrader(Trader trader);

//	/**
//	 * デフォルトの注文方法を返します。
//	 * 
//	 * @return デフォルトの注文方法
//	 */
//	public OrderTiming getDefaultOrderTiming();

	/**
	 * デフォルトの注文方法を設定します。
	 * 
	 * @param defaultOrderTiming デフォルトの注文方法
	 */
	public void setDefaultOrderTiming(OrderTiming defaultOrderTiming);

//	/**
//	 * シンボルを返します。
//	 * 
//	 * @return シンボル
//	 */
//	public String getSymbol();

	/**
	 * シンボルを設定します。
	 * 
	 * @param symbol シンボル
	 */
	public void setSymbol(String symbol);

	/**
	 * データセットを設定します。
	 * 
	 * @param dataset データセット
	 */
	public void setDataset(Dataset dataset);

	/**
	 * 開始データのインデックスを設定します。
	 * 
	 * @param index 開始データのインデックス
	 */
	public void setStartDataIndex(int index);

	/**
	 * 終了データのインデックスを設定します。
	 * 
	 * @param index 終了データのインデックス
	 */
	public void setEndDataIndex(int index);

	/**
	 * 当日データのインデックスを設定します。
	 * 
	 * @param index 当日データのインデックス
	 */
	public void setCurrentDataIndex(int index);

	/**
	 * 当日データの日時を設定します。
	 * 
	 * @param date 当日データの日時
	 */
	public void setDate(Date date);

	/**
	 * ストラテジーの名前を返します。
	 * 
	 * @return ストラテジーの名前
	 */
	public String getStrategyName();

	/**
	 * <p>ストラテジーを初期化します。</p>
	 */
	public void prepare();

	/**
	 * <p>ストラテジーを処理します。</p>
	 * 
	 * @return 処理を継続する場合は <code>false</code>。それ以外の場合は <code>true</code>。
	 * @throws Exception 例外が発生した場合
	 */
	public boolean execute() throws Exception;

	/**
	 * <p>ストラテジーを終了します。</p>
	 */
	public void close();

}
